@@ -14,10 +14,12 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: BankRomCheat.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: BankRomCheat.cxx 2155 2010-10-18 18:39:57Z stephena $
 //============================================================================
 
 #include "Console.hxx"
+#include "Cart.hxx"
+#include "OSystem.hxx"
 #include "BankRomCheat.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
