@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBufferGL.cxx 2003 2010-04-11 13:44:22Z stephena $
+// $Id: FrameBufferGL.cxx 2155 2010-10-18 18:39:57Z stephena $
 //============================================================================
 
 #ifdef DISPLAY_OPENGL
@@ -23,6 +23,7 @@
 #include <SDL_syswm.h>
 #include <sstream>
 #include <time.h>
+#include <fstream>
 
 #include "bspf.hxx"
 
@@ -292,10 +293,14 @@ bool FrameBufferGL::setVidMode(VideoMode& mode)
   if(!inUIMode)
   {
     // Aspect ratio (depends on whether NTSC or PAL is detected)
-    const string& frate = myOSystem->console().about().InitialFrameRate;
-    int aspect =
-      myOSystem->settings().getInt(frate == "60" ? "gl_aspectn" : "gl_aspectp");
-    mode.image_w = (uInt16)(float(mode.image_w * aspect) / 100.0);
+    // Not available in 'small' resolutions
+    if(myOSystem->desktopWidth() >= 640)
+    {
+      const string& frate = myOSystem->console().about().InitialFrameRate;
+      int aspect =
+        myOSystem->settings().getInt(frate == "60" ? "gl_aspectn" : "gl_aspectp");
+      mode.image_w = (uInt16)(float(mode.image_w * aspect) / 100.0);
+    }
 
     // Fullscreen mode stretching
     if(fullScreen() && myOSystem->settings().getBool("gl_fsmax") &&
@@ -437,18 +442,13 @@ cerr << "dimensions: " << (fullScreen() ? "(full)" : "") << endl
     myTiaSurface->setFilter(myOSystem->settings().getString("gl_filter"));
   }
 
-  // Make sure any old parts of the screen are erased
-  p_glClear(GL_COLOR_BUFFER_BIT);
-  SDL_GL_SwapBuffers();
-  p_glClear(GL_COLOR_BUFFER_BIT);
-
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBufferGL::invalidate()
 {
-// TODO - add code for this
+  p_glClear(GL_COLOR_BUFFER_BIT);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1649,7 +1649,7 @@ GLuint FBSurfaceGL::genShader(ShaderType type)
   // These shader files are stored in 'BASEDIR/shaders/'
   char* buffer = NULL;
   const string& filename =
-    myFB.myOSystem->baseDir() + BSPF_PATH_SEPARATOR + "shaders" +
+    myFB.myOSystem->baseDir() + "shaders" +
     BSPF_PATH_SEPARATOR + fFile;
   ifstream in(filename.c_str());
   if(in && in.is_open())
