@@ -14,13 +14,14 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CpuDebug.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: CpuDebug.cxx 2183 2010-11-11 16:46:41Z stephena $
 //============================================================================
 
 #include <sstream>
 
 #include "Array.hxx"
 #include "M6502.hxx"
+#include "Debugger.hxx"
 #include "CartDebug.hxx"
 #include "TIADebug.hxx"
 
@@ -42,6 +43,11 @@ const DebuggerState& CpuDebug::getState()
   myState.X  = mySystem.m6502().X;
   myState.Y  = mySystem.m6502().Y;
 
+  myState.srcS = mySystem.m6502().lastSrcAddressS();
+  myState.srcA = mySystem.m6502().lastSrcAddressA();
+  myState.srcX = mySystem.m6502().lastSrcAddressX();
+  myState.srcY = mySystem.m6502().lastSrcAddressY();
+
   Debugger::set_bits(myState.PS, myState.PSbits);
 
   return myState;
@@ -57,6 +63,11 @@ void CpuDebug::saveOldState()
   myOldState.X  = mySystem.m6502().X;
   myOldState.Y  = mySystem.m6502().Y;
 
+  myOldState.srcS = mySystem.m6502().lastSrcAddressS();
+  myOldState.srcA = mySystem.m6502().lastSrcAddressA();
+  myOldState.srcX = mySystem.m6502().lastSrcAddressX();
+  myOldState.srcY = mySystem.m6502().lastSrcAddressY();
+
   Debugger::set_bits(myOldState.PS, myOldState.PSbits);
 }
 
@@ -64,6 +75,7 @@ void CpuDebug::saveOldState()
 string CpuDebug::toString()
 {
   // TODO - this doesn't seem to be used anywhere ??
+  //        if it's ever used, convert to C++ stringstream
   string result;
   char buf[255];
 
