@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Debugger.cxx 2005 2010-04-12 22:26:41Z stephena $
+// $Id: Debugger.cxx 2155 2010-10-18 18:39:57Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -37,6 +37,7 @@
 #include "System.hxx"
 #include "M6502.hxx"
 #include "Cart.hxx"
+#include "TIA.hxx"
 
 #include "CartDebug.hxx"
 #include "CpuDebug.hxx"
@@ -50,7 +51,7 @@
 
 #include "RomWidget.hxx"
 #include "Expression.hxx"
-
+#include "PackedBitArray.hxx"
 #include "YaccParser.hxx"
 
 #include "Debugger.hxx"
@@ -80,10 +81,10 @@ static const char* builtin_functions[][3] = {
   { "_reset", "!(*SWCHB & $01)", "Game Reset pressed" },
   { "_color", "*SWCHB & $08", "Color/BW set to Color" },
   { "_bw", "!(*SWCHB & $08)", "Color/BW set to BW" },
-  { "_diff0b", "!(*SWCHB & $40)", "Left difficulty set to B (easy)" },
-  { "_diff0a", "*SWCHB & $40", "Left difficulty set to A (hard)" },
-  { "_diff1b", "!(*SWCHB & $80)", "Right difficulty set to B (easy)" },
-  { "_diff1a", "*SWCHB & $80", "Right difficulty set to A (hard)" },
+  { "_diff0b", "!(*SWCHB & $40)", "Left diff. set to B (easy)" },
+  { "_diff0a", "*SWCHB & $40", "Left diff. set to A (hard)" },
+  { "_diff1b", "!(*SWCHB & $80)", "Right diff. set to B (easy)" },
+  { "_diff1a", "*SWCHB & $80", "Right diff. set to A (hard)" },
 
   // empty string marks end of list, do not remove
   { 0, 0, 0 }
@@ -127,15 +128,6 @@ Debugger::Debugger(OSystem* osystem)
     myHeight(620),
     myRewindManager(NULL)
 {
-  // Get the dialog size
-  int w, h;
-  myOSystem->settings().getSize("debuggerres", w, h);
-  myWidth = BSPF_max(w, 0);
-  myHeight = BSPF_max(h, 0);
-  myWidth = BSPF_max(myWidth, 1050u);
-  myHeight = BSPF_max(myHeight, 620u);
-  myOSystem->settings().setSize("debuggerres", myWidth, myHeight);
-
   // Init parser
   myParser = new DebuggerParser(this);
   myBreakPoints = new PackedBitArray(0x10000);
@@ -169,6 +161,15 @@ Debugger::~Debugger()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::initialize()
 {
+  // Get the dialog size
+  int w, h;
+  myOSystem->settings().getSize("debuggerres", w, h);
+  myWidth = BSPF_max(w, 0);
+  myHeight = BSPF_max(h, 0);
+  myWidth = BSPF_max(myWidth, 1050u);
+  myHeight = BSPF_max(myHeight, 620u);
+  myOSystem->settings().setSize("debuggerres", myWidth, myHeight);
+
   const GUI::Rect& r = getDialogBounds();
 
   delete myBaseDialog;
@@ -187,7 +188,7 @@ void Debugger::initialize()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::initializeVideo()
+FBInitStatus Debugger::initializeVideo()
 {
   const GUI::Rect& r = getDialogBounds();
 
@@ -209,10 +210,7 @@ void Debugger::setConsole(Console* console)
   myCpuDebug = new CpuDebug(*this, *myConsole);
 
   delete myCartDebug;
-  // Register any RAM areas in the Cartridge
-  // Zero-page RAM is automatically recognized by CartDebug
-  myCartDebug = new CartDebug(*this, *myConsole, myConsole->cartridge().ramAreas());
-  myCartDebug->loadSymbolFile(myOSystem->romFile());
+  myCartDebug = new CartDebug(*this, *myConsole, *myOSystem);
 
   delete myRiotDebug;
   myRiotDebug = new RiotDebug(*this, *myConsole);
@@ -254,8 +252,7 @@ string Debugger::autoExec()
   ostringstream buf;
 
   // autoexec.stella is always run
-  FilesystemNode autoexec(myOSystem->baseDir() + BSPF_PATH_SEPARATOR +
-                          "autoexec.stella");
+  FilesystemNode autoexec(myOSystem->baseDir() + "autoexec.stella");
   buf << "autoExec():" << endl
       << myParser->exec(autoexec) << endl;
 
@@ -374,13 +371,21 @@ const string Debugger::setRAM(IntArray& args)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::saveState(int state)
 {
+  mySystem->clearDirtyPages();
+
+  unlockBankswitchState();
   myOSystem->state().saveState(state);
+  lockBankswitchState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::loadState(int state)
 {
+  mySystem->clearDirtyPages();
+
+  unlockBankswitchState();
   myOSystem->state().loadState(state);
+  lockBankswitchState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -528,7 +533,13 @@ void Debugger::nextFrame(int frames)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::rewindState()
 {
-  return myRewindManager->rewindState();
+  mySystem->clearDirtyPages();
+
+  unlockBankswitchState();
+  bool result = myRewindManager->rewindState();
+  lockBankswitchState();
+
+  return result;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -567,9 +578,9 @@ bool Debugger::setBank(int bank)
   if(myConsole->cartridge().bankCount() > 1)
   {
     myConsole->cartridge().unlockBank();
-    myConsole->cartridge().bank(bank);
+    bool status = myConsole->cartridge().bank(bank);
     myConsole->cartridge().lockBank();
-    return true;
+    return status;
   }
   return false;
 }
@@ -603,6 +614,9 @@ void Debugger::setStartState()
 
   // Save initial state, but don't add it to the rewind list
   saveOldState(false);
+
+  // Set the 're-disassemble' flag, but don't do it until the next scheduled time 
+  myRom->invalidate(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -756,7 +770,7 @@ string Debugger::builtinHelp() const
     if(len > i_maxlen)  i_maxlen = len;
   }
 
-  buf << endl << "Built-in functions:" << endl;
+  buf << setfill(' ') << endl << "Built-in functions:" << endl;
   for(int i = 0; builtin_functions[i][0] != 0; ++i)
   {
     buf << setw(c_maxlen) << left << builtin_functions[i][0]
