@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AudioWidget.cxx,v 1.7 2008/02/06 13:45:20 stephena Exp $
+// $Id: AudioWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -101,29 +102,7 @@ AudioWidget::~AudioWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
-/* FIXME - implement this
-  // We simply change the values in the DataGridWidget
-  // It will then send the 'kDGItemDataChangedCmd' signal to change the actual
-  // memory location
-  int addr, value;
-  string buf;
-
-  Debugger& dbg = instance()->debugger();
-  TIADebug& tia = dbg.tiaDebug();
-
-  switch(cmd)
-  {
-    case kDGItemDataChangedCmd:
-      switch(id)
-      {
-        case kNusizP0ID:
-          tia.nusizP0(myNusizP0->getSelectedValue());
-          myNusizP0Text->setEditString(tia.nusizP0String());
-          break;
-      }
-      break;
-  }
-*/
+  // TODO - implement this
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -140,7 +119,7 @@ void AudioWidget::fillGrid()
   IntArray vlist;
   BoolArray blist, changed, grNew, grOld;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
   TiaState state    = (TiaState&) tia.getState();
   TiaState oldstate = (TiaState&) tia.getOldState();
