@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DataGridWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: DataGridWidget.cxx 2155 2010-10-18 18:39:57Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -26,7 +26,9 @@
 #include "Debugger.hxx"
 #include "FrameBuffer.hxx"
 #include "DataGridWidget.hxx"
+#include "DataGridOpsWidget.hxx"
 #include "RamWidget.hxx"
+#include "ScrollBarWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
@@ -593,6 +595,12 @@ GUI::Rect DataGridWidget::getEditRect() const
   return r;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int DataGridWidget::getWidth() const
+{
+  return _w + (_scrollBar ? kScrollBarWidth : 0);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::startEditMode()
 {
@@ -637,7 +645,7 @@ bool DataGridWidget::tryInsertChar(char c, int pos)
   // Not sure how efficient this is, or should we even care?
   c = tolower(c);
   if((c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') ||
-     c == '%' || c == '#' || c == '$')
+     c == '\\' || c == '#' || c == '$')
   {
     _editString.insert(pos, 1, c);
     return true;
