@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3E.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: Cart3E.cxx 2184 2010-11-11 22:24:51Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -25,14 +25,17 @@
 #include "Cart3E.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge3E::Cartridge3E(const uInt8* image, uInt32 size)
-  : mySize(size)
+Cartridge3E::Cartridge3E(const uInt8* image, uInt32 size,
+                         const Settings& settings)
+  : Cartridge(settings),
+    mySize(size)
 {
   // Allocate array for the ROM image
   myImage = new uInt8[mySize];
 
   // Copy the ROM image into my buffer
   memcpy(myImage, image, mySize);
+  createCodeAccessBase(mySize + 32768);
 
   // This cart can address a 1024 byte bank of RAM @ 0x1000
   // However, it may not be addressable all the time (it may be swapped out)
@@ -52,9 +55,12 @@ Cartridge3E::~Cartridge3E()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::reset()
 {
-  // Initialize RAM with random values
-  for(uInt32 i = 0; i < 32768; ++i)
-    myRam[i] = mySystem->randGenerator().next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 32768; ++i)
+      myRam[i] = mySystem->randGenerator().next();
+  else
+    memset(myRam, 0, 32768);
 
   // We'll map the startup bank into the first segment upon reset
   bank(myStartBank);
@@ -70,26 +76,21 @@ void Cartridge3E::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1800 & mask) == 0);
 
-  System::PageAccess access;
+  System::PageAccess access(0, 0, 0, this, System::PA_READWRITE);
 
   // Set the page accessing methods for the hot spots (for 100% emulation
   // we need to chain any accesses below 0x40 to the TIA. Our poke() method
   // does this via mySystem->tiaPoke(...), at least until we come up with a
   // cleaner way to do it).
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READWRITE;
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
     mySystem->setPageAccess(i >> shift, access);
 
   // Setup the second segment to always point to the last ROM slice
-  access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;
   for(uInt32 j = 0x1800; j < 0x2000; j += (1 << shift))
   {
     access.directPeekBase = &myImage[(mySize - 2048) + (j & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[(mySize - 2048) + (j & 0x07FF)];
     mySystem->setPageAccess(j >> shift, access);
   }
 
@@ -158,9 +159,9 @@ bool Cartridge3E::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge3E::bank(uInt16 bank)
+bool Cartridge3E::bank(uInt16 bank)
 { 
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   if(bank < 256)
   {
@@ -180,15 +181,13 @@ void Cartridge3E::bank(uInt16 bank)
     uInt16 shift = mySystem->pageShift();
   
     // Setup the page access methods for the current bank
-    System::PageAccess access;
-    access.directPokeBase = 0;
-    access.device = this;
-    access.type = System::PA_READ;
+    System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
     // Map ROM image into the system
     for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
     {
       access.directPeekBase = &myImage[offset + (address & 0x07FF)];
+      access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x07FF)];
       mySystem->setPageAccess(address >> shift, access);
     }
   }
@@ -203,15 +202,13 @@ void Cartridge3E::bank(uInt16 bank)
     uInt32 address;
   
     // Setup the page access methods for the current bank
-    System::PageAccess access;
-    access.directPokeBase = 0;
-    access.device = this;
-    access.type = System::PA_READ;
+    System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
     // Map read-port RAM image into the system
     for(address = 0x1000; address < 0x1400; address += (1 << shift))
     {
       access.directPeekBase = &myRam[offset + (address & 0x03FF)];
+      access.codeAccessBase = &myCodeAccessBase[mySize + offset + (address & 0x03FF)];
       mySystem->setPageAccess(address >> shift, access);
     }
 
@@ -222,10 +219,11 @@ void Cartridge3E::bank(uInt16 bank)
     for(address = 0x1400; address < 0x1800; address += (1 << shift))
     {
       access.directPokeBase = &myRam[offset + (address & 0x03FF)];
+      access.codeAccessBase = &myCodeAccessBase[mySize + offset + (address & 0x03FF)];
       mySystem->setPageAccess(address >> shift, access);
     }
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -273,11 +271,9 @@ const uInt8* Cartridge3E::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3E::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
+    out.putString(name());
     out.putInt(myCurrentBank);
 
     // Output RAM
@@ -297,11 +293,9 @@ bool Cartridge3E::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3E::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16) in.getInt();
