@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3F.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: Cart3F.cxx 2138 2010-10-03 00:23:13Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -25,14 +25,17 @@
 #include "Cart3F.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge3F::Cartridge3F(const uInt8* image, uInt32 size)
-  : mySize(size)
+Cartridge3F::Cartridge3F(const uInt8* image, uInt32 size,
+                         const Settings& settings)
+  : Cartridge(settings),
+    mySize(size)
 {
   // Allocate array for the ROM image
   myImage = new uInt8[mySize];
 
   // Copy the ROM image into my buffer
   memcpy(myImage, image, mySize);
+  createCodeAccessBase(mySize);
 
   // Remember startup bank
   myStartBank = 0;
@@ -61,25 +64,21 @@ void Cartridge3F::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1800 & mask) == 0);
 
+  System::PageAccess access(0, 0, 0, this, System::PA_READWRITE);
+
   // Set the page accessing methods for the hot spots (for 100% emulation
   // we need to chain any accesses below 0x40 to the TIA. Our poke() method
   // does this via mySystem->tiaPoke(...), at least until we come up with a
   // cleaner way to do it).
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READWRITE;
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
     mySystem->setPageAccess(i >> shift, access);
 
   // Setup the second segment to always point to the last ROM slice
-  access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;
   for(uInt32 j = 0x1800; j < 0x2000; j += (1 << shift))
   {
     access.directPeekBase = &myImage[(mySize - 2048) + (j & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[(mySize - 2048) + (j & 0x07FF)];
     mySystem->setPageAccess(j >> shift, access);
   }
 
@@ -123,9 +122,9 @@ bool Cartridge3F::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge3F::bank(uInt16 bank)
+bool Cartridge3F::bank(uInt16 bank)
 { 
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Make sure the bank they're asking for is reasonable
   if(((uInt32)bank << 11) < mySize)
@@ -143,18 +142,16 @@ void Cartridge3F::bank(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Map ROM image into the system
   for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x07FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -192,11 +189,9 @@ const uInt8* Cartridge3F::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3F::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
+    out.putString(name());
     out.putInt(myCurrentBank);
   }
   catch(const char* msg)
@@ -211,11 +206,9 @@ bool Cartridge3F::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3F::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16) in.getInt();
