@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF6SC.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartF6SC.cxx 2145 2010-10-10 20:24:22Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,10 +24,12 @@
 #include "CartF6SC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeF6SC::CartridgeF6SC(const uInt8* image)
+CartridgeF6SC::CartridgeF6SC(const uInt8* image, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image, 16384);
+  createCodeAccessBase(16384);
 
   // This cart contains 128 bytes extended RAM @ 0x1000
   registerRamArea(0x1000, 128, 0x80, 0x00);
@@ -44,9 +46,12 @@ CartridgeF6SC::~CartridgeF6SC()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF6SC::reset()
 {
-  // Initialize RAM with random values
-  for(uInt32 i = 0; i < 128; ++i)
-    myRAM[i] = mySystem->randGenerator().next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 128; ++i)
+      myRAM[i] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 128);
 
   // Upon reset we switch to the startup bank
   bank(myStartBank);
@@ -62,33 +67,24 @@ void CartridgeF6SC::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1080 & mask) == 0) && ((0x1100 & mask) == 0));
 
-  System::PageAccess access;
-
-  // Set the page accessing methods for the hot spots
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
-  for(uInt32 i = (0x1FF6 & ~mask); i < 0x2000; i += (1 << shift))
-    mySystem->setPageAccess(i >> shift, access);
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Set the page accessing method for the RAM writing pages
-  access.directPeekBase = 0;
-  access.device = this;
   access.type = System::PA_WRITE;
   for(uInt32 j = 0x1000; j < 0x1080; j += (1 << shift))
   {
     access.directPokeBase = &myRAM[j & 0x007F];
+    access.codeAccessBase = &myCodeAccessBase[j & 0x007F];
     mySystem->setPageAccess(j >> shift, access);
   }
 
   // Set the page accessing method for the RAM reading pages
   access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;
   for(uInt32 k = 0x1080; k < 0x1100; k += (1 << shift))
   {
     access.directPeekBase = &myRAM[k & 0x007F];
+    access.codeAccessBase = &myCodeAccessBase[0x80 + (k & 0x007F)];
     mySystem->setPageAccess(k >> shift, access);
   }
 
@@ -185,9 +181,9 @@ bool CartridgeF6SC::poke(uInt16 address, uInt8)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeF6SC::bank(uInt16 bank)
+bool CartridgeF6SC::bank(uInt16 bank)
 { 
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -195,20 +191,24 @@ void CartridgeF6SC::bank(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
-  // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
-  // Map ROM image into the system
+  // Set the page accessing methods for the hot spots
+  for(uInt32 i = (0x1FF6 & ~mask); i < 0x2000; i += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[offset + (i & 0x0FFF)];
+    mySystem->setPageAccess(i >> shift, access);
+  }
+
+  // Setup the page access methods for the current bank
   for(uInt32 address = 0x1100; address < (0x1FF6U & ~mask);
       address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -251,12 +251,9 @@ const uInt8* CartridgeF6SC::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeF6SC::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
-
+    out.putString(name());
     out.putInt(myCurrentBank);
 
     // The 128 bytes of RAM
@@ -277,11 +274,9 @@ bool CartridgeF6SC::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeF6SC::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16) in.getInt();
