@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartFA.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartFA.cxx 2145 2010-10-10 20:24:22Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,10 +24,12 @@
 #include "CartFA.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeFA::CartridgeFA(const uInt8* image)
+CartridgeFA::CartridgeFA(const uInt8* image, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image, 12288);
+  createCodeAccessBase(12288);
 
   // This cart contains 256 bytes extended RAM @ 0x1000
   registerRamArea(0x1000, 256, 0x100, 0x00);
@@ -44,9 +46,12 @@ CartridgeFA::~CartridgeFA()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFA::reset()
 {
-  // Initialize RAM with random values
-  for(uInt32 i = 0; i < 256; ++i)
-    myRAM[i] = mySystem->randGenerator().next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 256; ++i)
+      myRAM[i] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 256);
 
   // Upon reset we switch to the startup bank
   bank(myStartBank);
@@ -62,33 +67,24 @@ void CartridgeFA::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1100 & mask) == 0) && ((0x1200 & mask) == 0));
 
-  System::PageAccess access;
-
-  // Set the page accessing methods for the hot spots
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
-  for(uInt32 i = (0x1FF8 & ~mask); i < 0x2000; i += (1 << shift))
-    mySystem->setPageAccess(i >> shift, access);
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Set the page accessing method for the RAM writing pages
-  access.directPeekBase = 0;
-  access.device = this;
   access.type = System::PA_WRITE;
   for(uInt32 j = 0x1000; j < 0x1100; j += (1 << shift))
   {
     access.directPokeBase = &myRAM[j & 0x00FF];
+    access.codeAccessBase = &myCodeAccessBase[j & 0x00FF];
     mySystem->setPageAccess(j >> shift, access);
   }
  
   // Set the page accessing method for the RAM reading pages
   access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;
   for(uInt32 k = 0x1100; k < 0x1200; k += (1 << shift))
   {
     access.directPeekBase = &myRAM[k & 0x00FF];
+    access.codeAccessBase = &myCodeAccessBase[0x100 + (k & 0x00FF)];
     mySystem->setPageAccess(k >> shift, access);
   }
 
@@ -175,9 +171,9 @@ bool CartridgeFA::poke(uInt16 address, uInt8)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeFA::bank(uInt16 bank)
+bool CartridgeFA::bank(uInt16 bank)
 {
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -185,20 +181,24 @@ void CartridgeFA::bank(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
-  // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
-  // Map ROM image into the system
+  // Set the page accessing methods for the hot spots
+  for(uInt32 i = (0x1FF8 & ~mask); i < 0x2000; i += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[offset + (i & 0x0FFF)];
+    mySystem->setPageAccess(i >> shift, access);
+  }
+
+  // Setup the page access methods for the current bank
   for(uInt32 address = 0x1200; address < (0x1FF8U & ~mask);
       address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -241,12 +241,9 @@ const uInt8* CartridgeFA::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeFA::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
-
+    out.putString(name());
     out.putInt(myCurrentBank);
 
     // The 256 bytes of RAM
@@ -266,11 +263,9 @@ bool CartridgeFA::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeFA::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16) in.getInt();
