@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBuffer.cxx 2029 2010-04-29 12:46:07Z stephena $
+// $Id: FrameBuffer.cxx 2169 2010-11-07 16:19:26Z stephena $
 //============================================================================
 
 #include <algorithm>
@@ -43,6 +43,7 @@
 FrameBuffer::FrameBuffer(OSystem* osystem)
   : myOSystem(osystem),
     myScreen(0),
+    mySDLFlags(0),
     myRedrawEntireFrame(true),
     myUsePhosphor(false),
     myPhosphorBlend(77),
@@ -66,7 +67,7 @@ FrameBuffer::~FrameBuffer(void)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
+FBInitStatus FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
 {
   ostringstream buf;
 
@@ -78,25 +79,50 @@ bool FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
     {
       buf << "ERROR: Couldn't initialize SDL: " << SDL_GetError() << endl;
       myOSystem->logMessage(buf.str(), 0);
-      return false;
+      return kFailComplete;
     }
   }
   myInitializedCount++;
 
+  // A 'windowed' system is defined as one where the window size can be
+  // larger than the screen size, as there's some sort of window manager
+  // that takes care of it (all current desktop systems fall in this category)
+  // However, some systems have no concept of windowing, and have hard limits
+  // on how large a window can be (ie, the size of the 'desktop' is the
+  // absolute upper limit on window size)
+  //
+  // If the WINDOWED_SUPPORT macro is defined, we treat the system as the
+  // former type; if not, as the latter type
+
+  uInt32 flags = mySDLFlags;
+#ifdef WINDOWED_SUPPORT
+  // We assume that a desktop size of at least 640x480 means that we're
+  // running on a 'large' system, and the window size requirements can
+  // be relaxed
+  // Otherwise, we treat the system as if WINDOWED_SUPPORT is not defined
+  if(myOSystem->desktopWidth() < 640 && myOSystem->desktopHeight() < 480 &&
+      (myOSystem->desktopWidth() < width || myOSystem->desktopHeight() < height))
+    return kFailTooLarge;
+
+  if(myOSystem->settings().getString("fullscreen") == "1")
+  {
+    if(myOSystem->desktopWidth() < width || myOSystem->desktopHeight() < height)
+      return kFailTooLarge;
+
+    flags |= SDL_FULLSCREEN;
+  }
+  else
+    flags &= ~SDL_FULLSCREEN;
+#else
   // Make sure this mode is even possible
   // We only really need to worry about it in non-windowed environments,
   // where requesting a window that's too large will probably cause a crash
-#ifndef WINDOWED_SUPPORT
   if(myOSystem->desktopWidth() < width || myOSystem->desktopHeight() < height)
-    return false;
+    return kFailTooLarge;
 #endif
 
-  // Initialize SDL flags and set fullscreen flag
-  // This must be done before any modes are initialized
-  mySDLFlags = 0;
-#ifdef WINDOWED_SUPPORT
-  if(myOSystem->settings().getString("fullscreen") == "1") mySDLFlags = SDL_FULLSCREEN;
-#endif
+  // Only update the actual flags if no errors were detected
+  mySDLFlags = flags;
 
   // Set the available video modes for this framebuffer
   setAvailableVidModes(width, height);
@@ -109,13 +135,10 @@ bool FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
     setWindowTitle(title);
     if(myInitializedCount == 1) setWindowIcon();
 
-    if(!initSubsystem(mode))
-    {
-      myOSystem->logMessage("ERROR: Couldn't initialize video subsystem\n", 0);
-      return false;
-    }
-    else
+    if(initSubsystem(mode))
     {
+      centerAppWindow(mode);
+
       myImageRect.setWidth(mode.image_w);
       myImageRect.setHeight(mode.image_h);
       myImageRect.moveTo(mode.image_x, mode.image_y);
@@ -129,9 +152,14 @@ bool FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
         myOSystem->settings().setString("fullscreen", fullScreen() ? "1" : "0");
       setCursorState();
     }
+    else
+    {
+      myOSystem->logMessage("ERROR: Couldn't initialize video subsystem\n", 0);
+      return kFailNotSupported;
+    }
   }
   else
-    return false;
+    return kFailTooLarge;
 
   // Enable unicode so we can see translated key events
   // (lowercase vs. uppercase characters)
@@ -161,7 +189,7 @@ bool FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
   if(myInitializedCount == 1)
     myOSystem->logMessage(about() + "\n", 1);
 
-  return true;
+  return kSuccess;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -420,54 +448,66 @@ void FrameBuffer::refresh()
   // This method is in essence a FULL refresh, putting all rendering
   // buffers in a known, fully redrawn state
 
-  invalidate();
   bool doubleBuffered = (type() == kGLBuffer);
   switch(myOSystem->eventHandler().state())
   {
     case EventHandler::S_EMULATE:
     case EventHandler::S_PAUSE:
+      invalidate();
       drawTIA(true);
       if(doubleBuffered)
+      {
+        postFrameUpdate();
+        invalidate();
         drawTIA(true);
+      }
       break;
 
     case EventHandler::S_MENU:
+      invalidate();
       drawTIA(true);
       myOSystem->menu().draw(true);
       if(doubleBuffered)
       {
         postFrameUpdate();
+        invalidate();
         drawTIA(true);
         myOSystem->menu().draw(true);
       }
       break;
 
     case EventHandler::S_CMDMENU:
+      invalidate();
       drawTIA(true);
       myOSystem->commandMenu().draw(true);
       if(doubleBuffered)
       {
         postFrameUpdate();
+        invalidate();
         drawTIA(true);
         myOSystem->commandMenu().draw(true);
       }
       break;
 
     case EventHandler::S_LAUNCHER:
+      invalidate();
       myOSystem->launcher().draw(true);
       if(doubleBuffered)
       {
         postFrameUpdate();
+        invalidate();
         myOSystem->launcher().draw(true);
       }
       break;
 
   #ifdef DEBUGGER_SUPPORT
     case EventHandler::S_DEBUGGER:
+      invalidate();
       myOSystem->debugger().draw(true);
       if(doubleBuffered)
       {
         postFrameUpdate();
+        invalidate();
         myOSystem->debugger().draw(true);
       }
       break;
@@ -657,6 +697,8 @@ bool FrameBuffer::changeVidMode(int direction)
   VideoMode vidmode = myCurrentModeList->current(myOSystem->settings(), fullScreen());
   if(setVidMode(vidmode))
   {
+    centerAppWindow(vidmode);
+
     myImageRect.setWidth(vidmode.image_w);
     myImageRect.setHeight(vidmode.image_h);
     myImageRect.moveTo(vidmode.image_x, vidmode.image_y);
@@ -976,6 +1018,20 @@ FrameBuffer::VideoMode FrameBuffer::getSavedVidMode()
   return myCurrentModeList->current(myOSystem->settings(), fullScreen());
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::centerAppWindow(const VideoMode& mode)
+{
+  // Attempt to center the application window in non-fullscreen mode
+  if(!fullScreen() && myOSystem->settings().getBool("center"))
+  {
+    int x = mode.screen_w >= myOSystem->desktopWidth() ? 0 : 
+      ((myOSystem->desktopWidth() - mode.screen_w) >> 1);
+    int y = mode.screen_h >= myOSystem->desktopHeight() ? 0 : 
+      ((myOSystem->desktopHeight() - mode.screen_h) >> 1);
+    myOSystem->setAppWindowPos(x, y, mode.screen_w, mode.screen_h);
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FrameBuffer::VideoModeList::VideoModeList()
   : myIdx(-1)
@@ -1027,7 +1083,7 @@ const FrameBuffer::VideoMode FrameBuffer::
   //   If it's 'auto', we just use the mode as already previously defined
   //   If it's not 'auto', attempt to fit the mode into the resolution
   //   specified by 'fullres' (if possible)
-  if(isFullscreen && BSPF_tolower(settings.getString("fullres")) != "auto")
+  if(isFullscreen && !BSPF_equalsIgnoreCase(settings.getString("fullres"), "auto"))
   {
     // Only use 'fullres' if it's *bigger* than the requested mode
     int w, h;
@@ -1085,8 +1141,8 @@ void FrameBuffer::VideoModeList::setByGfxMode(const string& name)
   GraphicsMode gfxmode;
   for(uInt32 i = 0; i < GFX_NumModes; ++i)
   {
-    if(ourGraphicsModes[i].name == BSPF_tolower(name) ||
-       ourGraphicsModes[i].description == BSPF_tolower(name))
+    if(BSPF_equalsIgnoreCase(ourGraphicsModes[i].name, name) ||
+       BSPF_equalsIgnoreCase(ourGraphicsModes[i].description, name))
     {
       gfxmode = ourGraphicsModes[i];
       found = true;
@@ -1146,21 +1202,27 @@ void FrameBuffer::VideoModeList::print()
   for(Common::Array<VideoMode>::const_iterator i = myModeList.begin();
       i != myModeList.end(); ++i)
   {
-    cerr << "  Mode " << i << endl
-         << "    screen w = " << i->screen_w << endl
-         << "    screen h = " << i->screen_h << endl
-         << "    image x  = " << i->image_x << endl
-         << "    image y  = " << i->image_y << endl
-         << "    image w  = " << i->image_w << endl
-         << "    image h  = " << i->image_h << endl
-         << "    gfx id   = " << i->gfxmode.type << endl
-         << "    gfx name = " << i->gfxmode.name << endl
-         << "    gfx desc = " << i->gfxmode.description << endl
-         << "    gfx zoom = " << i->gfxmode.zoom << endl
-         << endl;
+    cerr << "  Mode " << i << endl;
+    print(*i);
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::VideoModeList::print(const VideoMode& mode)
+{
+  cerr << "    screen w = " << mode.screen_w << endl
+       << "    screen h = " << mode.screen_h << endl
+       << "    image x  = " << mode.image_x << endl
+       << "    image y  = " << mode.image_y << endl
+       << "    image w  = " << mode.image_w << endl
+       << "    image h  = " << mode.image_h << endl
+       << "    gfx id   = " << mode.gfxmode.type << endl
+       << "    gfx name = " << mode.gfxmode.name << endl
+       << "    gfx desc = " << mode.gfxmode.description << endl
+       << "    gfx zoom = " << mode.gfxmode.zoom << endl
+       << endl;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurface::box(uInt32 x, uInt32 y, uInt32 w, uInt32 h,
                     uInt32 colorA, uInt32 colorB)
