@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6502.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: M6502.cxx 2177 2010-11-10 14:01:41Z stephena $
 //============================================================================
 
 //#define DEBUG_OUTPUT
@@ -23,6 +23,26 @@
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
   #include "Expression.hxx"
+  #include "CartDebug.hxx"
+  #include "PackedBitArray.hxx"
+
+  // Flags for disassembly types
+  #define DISASM_SKIP  CartDebug::SKIP
+  #define DISASM_CODE  CartDebug::CODE
+  #define DISASM_GFX   CartDebug::GFX
+  #define DISASM_PGFX  CartDebug::PGFX
+  #define DISASM_DATA  CartDebug::DATA
+  #define DISASM_ROW   CartDebug::ROW
+  #define DISASM_NONE  0
+#else
+  // Flags for disassembly types
+  #define DISASM_SKIP  0
+  #define DISASM_CODE  0
+  #define DISASM_GFX   0
+  #define DISASM_PGFX  0
+  #define DISASM_DATA  0
+  #define DISASM_ROW   0
+  #define DISASM_NONE  0
 #endif
 
 #include "M6502.hxx"
@@ -37,7 +57,12 @@ M6502::M6502(uInt32 systemCyclesPerProcessorCycle)
     myNumberOfDistinctAccesses(0),
     myLastAddress(0),
     myLastPeekAddress(0),
-    myLastPokeAddress(0)
+    myLastPokeAddress(0),
+    myLastSrcAddressS(0),
+    myLastSrcAddressA(0),
+    myLastSrcAddressX(0),
+    myLastSrcAddressY(0),
+    myDataAddressForPoke(0)
 {
 #ifdef DEBUGGER_SUPPORT
   myDebugger    = NULL;
@@ -96,6 +121,11 @@ void M6502::reset()
   PC = (uInt16)mySystem->peek(0xfffc) | ((uInt16)mySystem->peek(0xfffd) << 8);
 
   myTotalInstructionCount = 0;
+
+  myLastAddress = myLastPeekAddress = myLastPokeAddress = 0;
+  myLastSrcAddressS = myLastSrcAddressA =
+    myLastSrcAddressX = myLastSrcAddressY = 0;
+  myDataAddressForPoke = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -152,7 +182,7 @@ void M6502::PS(uInt8 ps)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-inline uInt8 M6502::peek(uInt16 address)
+inline uInt8 M6502::peek(uInt16 address, uInt8 flags)
 {
   if(address != myLastAddress)
   {
@@ -168,9 +198,10 @@ inline uInt8 M6502::peek(uInt16 address)
     myHitTrapInfo.message = "RTrap: ";
     myHitTrapInfo.address = address;
   }
+//cerr << "addr = " << HEX4 << address << ", flags = " << Debugger::to_bin_8(flags) << endl;
 #endif
 
-  uInt8 result = mySystem->peek(address);
+  uInt8 result = mySystem->peek(address, flags);
   myLastAccessWasRead = true;
   myLastPeekAddress = address;
   return result;
@@ -238,14 +269,14 @@ bool M6502::execute(uInt32 number)
           return true;
       }
 #endif
-      uInt16 operandAddress = 0;
+      uInt16 operandAddress = 0, intermediateAddress = 0;
       uInt8 operand = 0;
 
       // Reset the peek/poke address pointers
-      myLastPeekAddress = myLastPokeAddress = 0;
+      myLastPeekAddress = myLastPokeAddress = myDataAddressForPoke = 0;
 
       // Fetch instruction at the program counter
-      IR = peek(PC++);
+      IR = peek(PC++, DISASM_CODE);  // This address represents a code section
 
 #ifdef DEBUG_OUTPUT
       debugStream << ::hex << setw(2) << (int)A << " "
@@ -256,7 +287,7 @@ bool M6502::execute(uInt32 number)
                   << setw(2) << (int)IR << "       "
 //      << "<" << ourAddressingModeTable[IR] << " ";
 //      debugStream << hex << setw(4) << operandAddress << " ";
-                  << setw(3) << ourInstructionMnemonicTable[IR]
+//                  << setw(3) << ourInstructionMnemonicTable[IR]
 
 //      debugStream << "PS=" << ::hex << setw(2) << (int)PS() << " ";
 
@@ -274,7 +305,6 @@ bool M6502::execute(uInt32 number)
           // Oops, illegal instruction executed so set fatal error flag
           myExecutionStatus |= FatalErrorBit;
       }
-
       myTotalInstructionCount++;
     }
 
@@ -365,9 +395,15 @@ bool M6502::save(Serializer& out) const
 
     // Indicates the number of distinct memory accesses
     out.putInt(myNumberOfDistinctAccesses);
-    // Indicates the last address which was accessed
+    // Indicates the last address(es) which was accessed
     out.putInt(myLastAddress);
-
+    out.putInt(myLastPeekAddress);
+    out.putInt(myLastPokeAddress);
+    out.putInt(myLastSrcAddressS);
+    out.putInt(myLastSrcAddressA);
+    out.putInt(myLastSrcAddressX);
+    out.putInt(myLastSrcAddressY);
+    out.putInt(myDataAddressForPoke);
   }
   catch(const char* msg)
   {
@@ -388,11 +424,11 @@ bool M6502::load(Serializer& in)
     if(in.getString() != CPU)
       return false;
 
-    A = (uInt8) in.getByte();    // Accumulator
-    X = (uInt8) in.getByte();    // X index register
-    Y = (uInt8) in.getByte();    // Y index register
-    SP = (uInt8) in.getByte();   // Stack Pointer
-    IR = (uInt8) in.getByte();   // Instruction register
+    A = (uInt8) in.getByte();   // Accumulator
+    X = (uInt8) in.getByte();   // X index register
+    Y = (uInt8) in.getByte();   // Y index register
+    SP = (uInt8) in.getByte();  // Stack Pointer
+    IR = (uInt8) in.getByte();  // Instruction register
     PC = (uInt16) in.getInt();  // Program Counter
 
     N = in.getBool();     // N flag for processor status register
@@ -407,8 +443,15 @@ bool M6502::load(Serializer& in)
 
     // Indicates the number of distinct memory accesses
     myNumberOfDistinctAccesses = (uInt32) in.getInt();
-    // Indicates the last address which was accessed
+    // Indicates the last address(es) which was accessed
     myLastAddress = (uInt16) in.getInt();
+    myLastPeekAddress = (uInt16) in.getInt();
+    myLastPokeAddress = (uInt16) in.getInt();
+    myLastSrcAddressS = (uInt16) in.getInt();
+    myLastSrcAddressA = (uInt16) in.getInt();
+    myLastSrcAddressX = (uInt16) in.getInt();
+    myLastSrcAddressY = (uInt16) in.getInt();
+    myDataAddressForPoke = (uInt16) in.getInt();
   }
   catch(const char* msg)
   {
