@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystem.cxx 2029 2010-04-29 12:46:07Z stephena $
+// $Id: OSystem.cxx 2168 2010-11-04 19:29:12Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -22,14 +22,15 @@
 #include <fstream>
 #include <zlib.h>
 
+#include <ctime>
 #ifdef HAVE_GETTIMEOFDAY
-  #include <time.h>
   #include <sys/time.h>
 #endif
 
 #include "bspf.hxx"
 
 #include "MediaFactory.hxx"
+#include "Sound.hxx"
 
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
@@ -51,6 +52,7 @@
 #include "FSNode.hxx"
 #include "unzip.h"
 #include "MD5.hxx"
+#include "Cart.hxx"
 #include "Settings.hxx"
 #include "PropsSet.hxx"
 #include "EventHandler.hxx"
@@ -95,6 +97,9 @@ OSystem::OSystem()
     myFont(NULL),
     myConsoleFont(NULL)
 {
+  // Calculate startup time
+  myMillisAtStart = (uInt32)(time(NULL) * 1000);
+
   // Get built-in features
   #ifdef DISPLAY_OPENGL
     myFeatures += "OpenGL ";
@@ -162,6 +167,7 @@ OSystem::~OSystem()
   delete myCommandMenu;
   delete myLauncher;
   delete myFont;
+  delete myInfoFont;
   delete mySmallFont;
   delete myConsoleFont;
   delete myLauncherFont;
@@ -221,18 +227,22 @@ bool OSystem::create()
   ////////////////////////////////////////////////////////////////////
   bool smallScreen = myDesktopWidth < 640 || myDesktopHeight < 480;
 
+  // The console font is always the same size (for now at least)
+  myConsoleFont  = new GUI::Font(GUI::consoleDesc);
+
   // This font is used in a variety of situations when a really small
   // font is needed; we let the specific widget/dialog decide when to
   // use it
   mySmallFont = new GUI::Font(GUI::stellaDesc);
 
-  // The console font is always the same size (for now at least)
-  myConsoleFont  = new GUI::Font(GUI::consoleDesc);
-
   // The general font used in all UI elements
   // This is determined by the size of the framebuffer
   myFont = new GUI::Font(smallScreen ? GUI::stellaDesc : GUI::stellaMediumDesc);
 
+  // The info font used in all UI elements
+  // This is determined by the size of the framebuffer
+  myInfoFont = new GUI::Font(smallScreen ? GUI::stellaDesc : GUI::consoleDesc);
+
   // The font used by the ROM launcher
   // Normally, this is configurable by the user, except in the case of
   // very small screens
@@ -303,44 +313,28 @@ void OSystem::setConfigPaths()
   FilesystemNode node;
   string s;
 
-  s = mySettings->getString("statedir");
-  if(s == "") s = myBaseDir + BSPF_PATH_SEPARATOR + "state";
-  node = FilesystemNode(s);
-  myStateDir = node.getPath();
-  mySettings->setString("statedir", node.getRelativePath());
-  if(!node.isDirectory())
-    AbstractFilesystemNode::makeDir(myStateDir);
+  validatePath("statedir", "state", myStateDir);
 
-  s = mySettings->getString("ssdir");
-  if(s == "") s = myBaseDir + BSPF_PATH_SEPARATOR + "snapshots";
-  node = FilesystemNode(s);
-  mySnapshotDir = node.getPath();
-  mySettings->setString("ssdir", node.getRelativePath());
-  if(!node.isDirectory())
-    AbstractFilesystemNode::makeDir(mySnapshotDir);
+  validatePath("ssdir", "snapshots", mySnapshotDir);
 
-  s = mySettings->getString("eepromdir");
-  if(s == "") s = myBaseDir;
-  node = FilesystemNode(s);
-  myEEPROMDir = node.getPath();
-  mySettings->setString("eepromdir", node.getRelativePath());
-  if(!node.isDirectory())
-    AbstractFilesystemNode::makeDir(myEEPROMDir);
+  validatePath("eepromdir", "", myEEPROMDir);
+
+  validatePath("cfgdir", "cfg", myCfgDir);
 
   s = mySettings->getString("cheatfile");
-  if(s == "") s = myBaseDir + BSPF_PATH_SEPARATOR + "stella.cht";
+  if(s == "") s = myBaseDir + "stella.cht";
   node = FilesystemNode(s);
   myCheatFile = node.getPath();
   mySettings->setString("cheatfile", node.getRelativePath());
 
   s = mySettings->getString("palettefile");
-  if(s == "") s = myBaseDir + BSPF_PATH_SEPARATOR + "stella.pal";
+  if(s == "") s = myBaseDir + "stella.pal";
   node = FilesystemNode(s);
   myPaletteFile = node.getPath();
   mySettings->setString("palettefile", node.getRelativePath());
 
   s = mySettings->getString("propsfile");
-  if(s == "") s = myBaseDir + BSPF_PATH_SEPARATOR + "stella.pro";
+  if(s == "") s = myBaseDir + "stella.pro";
   node = FilesystemNode(s);
   myPropertiesFile = node.getPath();
   mySettings->setString("propsfile", node.getRelativePath());
@@ -361,7 +355,10 @@ void OSystem::setBaseDir(const string& basedir)
   FilesystemNode node(basedir);
   myBaseDir = node.getPath();
   if(!node.isDirectory())
+  {
     AbstractFilesystemNode::makeDir(myBaseDir);
+    myBaseDir = FilesystemNode(node.getPath()).getPath();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -382,7 +379,7 @@ void OSystem::setFramerate(float framerate)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::createFrameBuffer()
+FBInitStatus OSystem::createFrameBuffer()
 {
   // There is only ever one FrameBuffer created per run of Stella
   // Due to the multi-surface nature of the FrameBuffer, repeatedly
@@ -397,24 +394,28 @@ bool OSystem::createFrameBuffer()
     myFrameBuffer = MediaFactory::createVideo(this);
 
   // Re-initialize the framebuffer to current settings
+  FBInitStatus fbstatus = kFailComplete;
   switch(myEventHandler->state())
   {
     case EventHandler::S_EMULATE:
     case EventHandler::S_PAUSE:
     case EventHandler::S_MENU:
     case EventHandler::S_CMDMENU:
-      if(!myConsole->initializeVideo())
+      fbstatus = myConsole->initializeVideo();
+      if(fbstatus != kSuccess)
         goto fallback;
       break;  // S_EMULATE, S_PAUSE, S_MENU, S_CMDMENU
 
     case EventHandler::S_LAUNCHER:
-      if(!myLauncher->initializeVideo())
+      fbstatus = myLauncher->initializeVideo();
+      if(fbstatus != kSuccess)
         goto fallback;
       break;  // S_LAUNCHER
 
 #ifdef DEBUGGER_SUPPORT
     case EventHandler::S_DEBUGGER:
-      if(!myDebugger->initializeVideo())
+      fbstatus = myDebugger->initializeVideo();
+      if(fbstatus != kSuccess)
         goto fallback;
       break;  // S_DEBUGGER
 #endif
@@ -434,28 +435,30 @@ bool OSystem::createFrameBuffer()
     setUIPalette();
   }
 
-  return true;
+  return fbstatus;
 
   // GOTO are normally considered evil, unless well documented :)
-  // If initialization of video system fails while in OpenGL mode,
-  // attempt to fallback to software mode
+  // If initialization of video system fails while in OpenGL mode
+  // because OpenGL is unavailable, attempt to fallback to software mode
+  // Otherwise, pass the error to the parent
 fallback:
-  if(myFrameBuffer && myFrameBuffer->type() == kGLBuffer)
+  if(fbstatus == kFailNotSupported && myFrameBuffer &&
+     myFrameBuffer->type() == kGLBuffer)
   {
     logMessage("ERROR: OpenGL mode failed, fallback to software\n", 0);
     delete myFrameBuffer; myFrameBuffer = NULL;
     mySettings->setString("video", "soft");
-    bool ret = createFrameBuffer();
-    if(ret)
+    FBInitStatus newstatus = createFrameBuffer();
+    if(newstatus == kSuccess)
     {
       setFramerate(60);
       myFrameBuffer->showMessage("OpenGL mode failed, fallback to software",
                                  kMiddleCenter, true);
     }
-    return ret;
+    return newstatus;
   }
   else
-    return false;
+    return fbstatus;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -519,7 +522,7 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
     //////////////////////////////////////////////////////////////////////////
     if(audiofirst)  myConsole->initializeAudio();
     myEventHandler->reset(EventHandler::S_EMULATE);
-    if(!createFrameBuffer())  // Takes care of initializeVideo()
+    if(createFrameBuffer() != kSuccess)  // Takes care of initializeVideo()
     {
       logMessage("ERROR: Couldn't create framebuffer for console\n", 0);
       myEventHandler->reset(EventHandler::S_LAUNCHER);
@@ -591,7 +594,7 @@ void OSystem::deleteConsole()
 bool OSystem::createLauncher()
 {
   myEventHandler->reset(EventHandler::S_LAUNCHER);
-  if(!createFrameBuffer())
+  if(createFrameBuffer() != kSuccess)
   {
     logMessage("ERROR: Couldn't create launcher\n", 0);
     return false;
@@ -753,10 +756,10 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
         if(strlen(filename) >= 4)
         {
           // Grab 3-character extension
-          char* ext = filename + strlen(filename) - 4;
+          const char* ext = filename + strlen(filename) - 4;
 
-          if(!BSPF_strcasecmp(ext, ".a26") || !BSPF_strcasecmp(ext, ".bin") ||
-             !BSPF_strcasecmp(ext, ".rom"))
+          if(BSPF_equalsIgnoreCase(ext, ".a26") || BSPF_equalsIgnoreCase(ext, ".bin") ||
+             BSPF_equalsIgnoreCase(ext, ".rom"))
           {
             file = filename;
             break;
@@ -779,7 +782,7 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
 
       // We don't have to check for any return errors from these functions,
       // since if there are, 'image' will not contain a valid ROM and the
-      // calling method can take of it
+      // calling method can take care of it
       unzOpenCurrentFile(tz);
       unzReadCurrentFile(tz, image, size);
       unzCloseCurrentFile(tz);
@@ -816,8 +819,8 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
   if(!myPropSet->getMD5(md5, props))
   {
     // Get the filename from the rom pathname
-    string::size_type pos = file.find_last_of("/\\");
-    if(pos != string::npos)  file = file.substr(pos+1);
+    FilesystemNode node(file);
+    file = node.getDisplayName();
 
     props.set(Cartridge_MD5, md5);
     props.set(Cartridge_Name, file);
@@ -853,47 +856,116 @@ void OSystem::resetLoopTiming()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setDefaultJoymap()
+void OSystem::validatePath(const string& setting, const string& partialpath,
+                           string& fullpath)
 {
-  EventMode mode;
-
-  mode = kEmulationMode;  // Default emulation events
-  // Left joystick (assume joystick zero, button zero)
-  myEventHandler->setDefaultJoyMapping(Event::JoystickZeroFire1, mode, 0, 0);
-  // Right joystick (assume joystick one, button zero)
-  myEventHandler->setDefaultJoyMapping(Event::JoystickOneFire1, mode, 1, 0);
+  const string& s = mySettings->getString(setting) != "" ?
+    mySettings->getString(setting) : myBaseDir + partialpath;
+  FilesystemNode node = FilesystemNode(s);
+  if(!node.isDirectory())
+  {
+    AbstractFilesystemNode::makeDir(s);
+    node = FilesystemNode(node.getPath());
+  }
+  fullpath = node.getPath();
+  mySettings->setString(setting, node.getRelativePath());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setDefaultJoyAxisMap()
+void OSystem::setDefaultJoymap(Event::Type event, EventMode mode)
 {
-  EventMode mode;
+#define SET_DEFAULT_BTN(sdb_event, sdb_mode, sdb_stick, sdb_button, sdb_cmp_event) \
+  if(eraseAll || sdb_cmp_event == sdb_event) \
+    myEventHandler->setDefaultJoyMapping(sdb_event, sdb_mode, sdb_stick, sdb_button);
 
-  mode = kEmulationMode;  // Default emulation events
-  // Left joystick left/right directions (assume joystick zero)
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickZeroLeft, mode, 0, 0, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickZeroRight, mode, 0, 0, 1);
-  // Left joystick up/down directions (assume joystick zero)
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickZeroUp, mode, 0, 1, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickZeroDown, mode, 0, 1, 1);
-  // Right joystick left/right directions (assume joystick one)
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickOneLeft, mode, 1, 0, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickOneRight, mode, 1, 0, 1);
-  // Right joystick left/right directions (assume joystick one)
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickOneUp, mode, 1, 1, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::JoystickOneDown, mode, 1, 1, 1);
+  bool eraseAll = (event == Event::NoType);
+  switch(mode)
+  {
+    case kEmulationMode:  // Default emulation events
+      // Left joystick (assume joystick zero, button zero)
+      SET_DEFAULT_BTN(Event::JoystickZeroFire1, mode, 0, 0, event);
+      // Right joystick (assume joystick one, button zero)
+      SET_DEFAULT_BTN(Event::JoystickOneFire1, mode, 1, 0, event);
+      break;
 
-  mode = kMenuMode;  // Default menu/UI events
-  myEventHandler->setDefaultJoyAxisMapping(Event::UILeft, mode, 0, 0, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::UIRight, mode, 0, 0, 1);
-  myEventHandler->setDefaultJoyAxisMapping(Event::UIUp, mode, 0, 1, 0);
-  myEventHandler->setDefaultJoyAxisMapping(Event::UIDown, mode, 0, 1, 1);
+    case kMenuMode:  // Default menu/UI events
+      // Left joystick (assume joystick zero, button zero)
+      SET_DEFAULT_BTN(Event::UISelect, mode, 0, 0, event);
+      // Right joystick (assume joystick one, button zero)
+      SET_DEFAULT_BTN(Event::UISelect, mode, 1, 0, event);
+      break;
+
+    default:
+      break;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setDefaultJoyHatMap()
+void OSystem::setDefaultJoyAxisMap(Event::Type event, EventMode mode)
 {
-// FIXME - add emul and UI events
+#define SET_DEFAULT_AXIS(sda_event, sda_mode, sda_stick, sda_axis, sda_val, sda_cmp_event) \
+  if(eraseAll || sda_cmp_event == sda_event) \
+    myEventHandler->setDefaultJoyAxisMapping(sda_event, sda_mode, sda_stick, sda_axis, sda_val);
+
+  bool eraseAll = (event == Event::NoType);
+  switch(mode)
+  {
+    case kEmulationMode:  // Default emulation events
+      // Left joystick left/right directions (assume joystick zero)
+      SET_DEFAULT_AXIS(Event::JoystickZeroLeft, mode, 0, 0, 0, event);
+      SET_DEFAULT_AXIS(Event::JoystickZeroRight, mode, 0, 0, 1, event);
+      // Left joystick up/down directions (assume joystick zero)
+      SET_DEFAULT_AXIS(Event::JoystickZeroUp, mode, 0, 1, 0, event);
+      SET_DEFAULT_AXIS(Event::JoystickZeroDown, mode, 0, 1, 1, event);
+      // Right joystick left/right directions (assume joystick one)
+      SET_DEFAULT_AXIS(Event::JoystickOneLeft, mode, 1, 0, 0, event);
+      SET_DEFAULT_AXIS(Event::JoystickOneRight, mode, 1, 0, 1, event);
+      // Right joystick left/right directions (assume joystick one)
+      SET_DEFAULT_AXIS(Event::JoystickOneUp, mode, 1, 1, 0, event);
+      SET_DEFAULT_AXIS(Event::JoystickOneDown, mode, 1, 1, 1, event);
+      break;
+
+    case kMenuMode:  // Default menu/UI events
+      SET_DEFAULT_AXIS(Event::UILeft, mode, 0, 0, 0, event);
+      SET_DEFAULT_AXIS(Event::UIRight, mode, 0, 0, 1, event);
+      SET_DEFAULT_AXIS(Event::UIUp, mode, 0, 1, 0, event);
+      SET_DEFAULT_AXIS(Event::UIDown, mode, 0, 1, 1, event);
+      break;
+
+    default:
+      break;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void OSystem::setDefaultJoyHatMap(Event::Type event, EventMode mode)
+{
+#define SET_DEFAULT_HAT(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir, sdh_cmp_event) \
+  if(eraseAll || sdh_cmp_event == sdh_event) \
+    myEventHandler->setDefaultJoyHatMapping(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir);
+
+  bool eraseAll = (event == Event::NoType);
+  switch(mode)
+  {
+    case kEmulationMode:  // Default emulation events
+      // Left joystick left/right directions (assume joystick zero and hat 0)
+      SET_DEFAULT_HAT(Event::JoystickZeroLeft, mode, 0, 0, EVENT_HATLEFT, event);
+      SET_DEFAULT_HAT(Event::JoystickZeroRight, mode, 0, 0, EVENT_HATRIGHT, event);
+      // Left joystick up/down directions (assume joystick zero and hat 0)
+      SET_DEFAULT_HAT(Event::JoystickZeroUp, mode, 0, 0, EVENT_HATUP, event);
+      SET_DEFAULT_HAT(Event::JoystickZeroDown, mode, 0, 0, EVENT_HATDOWN, event);
+      break;
+
+    case kMenuMode:  // Default menu/UI events
+      SET_DEFAULT_HAT(Event::UILeft, mode, 0, 0, EVENT_HATLEFT, event);
+      SET_DEFAULT_HAT(Event::UIRight, mode, 0, 0, EVENT_HATRIGHT, event);
+      SET_DEFAULT_HAT(Event::UIUp, mode, 0, 0, EVENT_HATUP, event);
+      SET_DEFAULT_HAT(Event::UIDown, mode, 0, 0, EVENT_HATDOWN, event);
+      break;
+
+    default:
+      break;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -918,12 +990,17 @@ void OSystem::stateChanged(EventHandler::State state)
 uInt64 OSystem::getTicks() const
 {
 #ifdef HAVE_GETTIMEOFDAY
+  // Gettimeofday natively refers to the UNIX epoch (a set time in the past)
   timeval now;
   gettimeofday(&now, 0);
 
   return uInt64(now.tv_sec) * 1000000 + now.tv_usec;
 #else
-  return uInt64(SDL_GetTicks()) * 1000;
+  // We use SDL_GetTicks, but add in the time when the application was
+  // initialized.  This is necessary, since SDL_GetTicks only measures how
+  // long SDL has been running, which can be the same between multiple runs
+  // of the application.
+  return uInt64(SDL_GetTicks() + myMillisAtStart) * 1000;
 #endif
 }
 
@@ -992,7 +1069,7 @@ bool OSystem::queryVideoHardware()
   // Normally, this wouldn't be set, and we ask SDL directly
   int w, h;
   mySettings->getSize("maxres", w, h);
-  if(w == 0 || h == 0)
+  if(w <= 0 || h <= 0)
   {
     const SDL_VideoInfo* info = SDL_GetVideoInfo();
     myDesktopWidth  = info->current_w;
@@ -1022,15 +1099,35 @@ bool OSystem::queryVideoHardware()
   }
   else
   {
+    // All modes must fit between the lower and upper limits of the desktop
+    // For 'small' desktop, this means larger than 320x240
+    // For 'large'/normal desktop, exclude all those less than 640x480
+    bool largeDesktop = myDesktopWidth >= 640 && myDesktopHeight >= 480;
+    uInt32 lowerWidth  = largeDesktop ? 640 : 320,
+           lowerHeight = largeDesktop ? 480 : 240;
     for(uInt32 i = 0; modes[i]; ++i)
+    {
+      if(modes[i]->w >= lowerWidth && modes[i]->w <= myDesktopWidth &&
+         modes[i]->h >= lowerHeight && modes[i]->h <= myDesktopHeight)
+      {
+        Resolution r;
+        r.width  = modes[i]->w;
+        r.height = modes[i]->h;
+        buf.str("");
+        buf << r.width << "x" << r.height;
+        r.name = buf.str();
+        myResolutions.insert_at(0, r);  // insert in opposite (of descending) order
+      }
+    }
+    // If no modes were valid, use the desktop dimensions
+    if(myResolutions.size() == 0)
     {
       Resolution r;
-      r.width  = modes[i]->w;
-      r.height = modes[i]->h;
-      buf.str("");
+      r.width  = myDesktopWidth;
+      r.height = myDesktopHeight;
       buf << r.width << "x" << r.height;
       r.name = buf.str();
-      myResolutions.insert_at(0, r);  // insert in opposite (of descending) order
+      myResolutions.push_back(r);
     }
   }
 
