@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx 2031 2010-05-03 12:01:43Z stephena $
+// $Id: Settings.cxx 2139 2010-10-03 18:19:09Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -53,7 +53,7 @@ Settings::Settings(OSystem* osystem)
   setInternal("tia_filter", "zoom2x");
   setInternal("fullscreen", "0");
   setInternal("fullres", "auto");
-  setInternal("center", "true");
+  setInternal("center", "false");
   setInternal("grabmouse", "false");
   setInternal("palette", "standard");
   setInternal("colorloss", "false");
@@ -80,7 +80,8 @@ Settings::Settings(OSystem* osystem)
   setInternal("joymap", "");
   setInternal("joyaxismap", "");
   setInternal("joyhatmap", "");
-  setInternal("joydeadzone", "0");
+  setInternal("combomap", "");
+  setInternal("joydeadzone", "13");
   setInternal("joyallow4", "false");
   setInternal("usemouse", "true");
   setInternal("pspeed", "6");
@@ -95,12 +96,13 @@ Settings::Settings(OSystem* osystem)
   setInternal("ssinterval", "2");
 
   // Config files and paths
-  setInternal("romdir", "~");
+  setInternal("romdir", "");
   setInternal("statedir", "");
   setInternal("cheatfile", "");
   setInternal("palettefile", "");
   setInternal("propsfile", "");
   setInternal("eepromdir", "");
+  setInternal("cfgdir", "");
 
   // ROM browser options
   setInternal("uselauncher", "true");
@@ -120,15 +122,18 @@ Settings::Settings(OSystem* osystem)
   setInternal("autoslot", "false");
   setInternal("showinfo", "1");
   setInternal("tiadriven", "false");
+  setInternal("ramrandom", "true");
   setInternal("avoxport", "");
   setInternal("stats", "false");
   setInternal("audiofirst", "true");
   setInternal("fastscbios", "false");
   setExternal("romloadcount", "0");
-  setExternal("maxres", "0x0");
+  setExternal("maxres", "");
 
   // Debugger options
   setInternal("resolvedata", "auto");
+  setInternal("gfxformat", "2");
+  setInternal("showaddr", "true");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -415,8 +420,14 @@ void Settings::usage()
     << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
     << "  -propsfile    <file>         Full pathname of ROM properties file\n"
     << "  -eepromdir    <dir>          Directory in which to save EEPROM files\n"
+    << "  -cfgdir       <dir>          Directory in which to save Distella config files\n"
     << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
     << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
+    << "  -holdreset                   Start the emulator with the Game Reset switch held down\n"
+    << "  -holdselect                  Start the emulator with the Game Select switch held down\n"
+    << "  -holdbutton0                 Start the emulator with the left joystick button held down\n"
+    << "  -tiadriven    <1|0>          Drive unused TIA pins randomly on a read/peek\n"
+    << "  -ramrandom    <1|0>          Randomize the contents of RAM on reset\n"
     << "  -help                        Show the text you're now reading\n"
   #ifdef DEBUGGER_SUPPORT
     << endl
@@ -425,13 +436,11 @@ void Settings::usage()
     << endl
     << "   -resolvedata  <never|       Set automatic code vs. data determination in the disassembler\n"
     << "                  always|auto>\n"
+    << "   -gfxformat    <2|16>        Set base to use for displaying GFX sections in the disassembler\n"
+    << "   -showaddr     <1|0>         Show opcode addresses in the disassembler\n"
     << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
     << "   -break        <address>     Set a breakpoint at 'address'\n"
     << "   -debug                      Start in debugger mode\n"
-    << "   -holdreset                  Start the emulator with the Game Reset switch held down\n"
-    << "   -holdselect                 Start the emulator with the Game Select switch held down\n"
-    << "   -holdbutton0                Start the emulator with the left joystick button held down\n"
-    << "   -tiadriven    <1|0>         Drive unused TIA pins randomly value on a read/peek\n"
     << endl
     << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
     << "   -type        <arg>          Same as using -bs\n"
