@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: StateManager.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: StateManager.cxx 2183 2010-11-11 16:46:41Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -22,6 +22,7 @@
 #include "OSystem.hxx"
 #include "Settings.hxx"
 #include "Console.hxx"
+#include "Cart.hxx"
 #include "Control.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
@@ -29,8 +30,8 @@
 
 #include "StateManager.hxx"
 
-#define STATE_HEADER "03010000state"
-#define MOVIE_HEADER "03010000movie"
+#define STATE_HEADER "03030000state"
+#define MOVIE_HEADER "03030000movie"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 StateManager::StateManager(OSystem* osystem)
@@ -61,7 +62,7 @@ bool StateManager::toggleRecordMode()
   {
     myActiveMode = kOffMode;
 
-    string moviefile = /*myOSystem->baseDir() + BSPF_PATH_SEPARATOR +*/ "test.inp";
+    string moviefile = /*myOSystem->baseDir() +*/ "test.inp";
     if(myMovieWriter.isOpen())
       myMovieWriter.close();
     if(!myMovieWriter.open(moviefile))
@@ -109,7 +110,7 @@ bool StateManager::toggleRewindMode()
   {
     myActiveMode = kOffMode;
 
-    string moviefile = /*myOSystem->baseDir() + BSPF_PATH_SEPARATOR +*/ "test.inp";
+    string moviefile = /*myOSystem->baseDir() + */ "test.inp";
     if(myMovieReader.isOpen())
       myMovieReader.close();
     if(!myMovieReader.open(moviefile))
@@ -178,7 +179,7 @@ void StateManager::loadState(int slot)
     if(slot < 0) slot = myCurrentSlot;
 
     ostringstream buf;
-    buf << myOSystem->stateDir() << BSPF_PATH_SEPARATOR
+    buf << myOSystem->stateDir()
         << myOSystem->console().properties().get(Cartridge_Name)
         << ".st" << slot;
 
@@ -222,7 +223,7 @@ void StateManager::saveState(int slot)
     if(slot < 0) slot = myCurrentSlot;
 
     ostringstream buf;
-    buf << myOSystem->stateDir() << BSPF_PATH_SEPARATOR
+    buf << myOSystem->stateDir()
         << myOSystem->console().properties().get(Cartridge_Name)
         << ".st" << slot;
 
