@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: System.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: System.cxx 2188 2010-11-12 18:54:08Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -212,21 +212,24 @@ void System::clearDirtyPages()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 System::peek(uInt16 addr)
+uInt8 System::peek(uInt16 addr, uInt8 flags)
 {
   PageAccess& access = myPageAccessTable[(addr & myAddressMask) >> myPageShift];
 
+#ifdef DEBUGGER_SUPPORT
+  // Set access type
+  if(access.codeAccessBase)
+    *(access.codeAccessBase + (addr & myPageMask)) |= flags;
+  else
+    access.device->setAccessFlags(addr, flags);
+#endif
+
+  // See if this page uses direct accessing or not 
   uInt8 result;
- 
-  // See if this page uses direct accessing or not 
-  if(access.directPeekBase != 0)
-  {
+  if(access.directPeekBase)
     result = *(access.directPeekBase + (addr & myPageMask));
-  }
   else
-  {
     result = access.device->peek(addr);
-  }
 
 #ifdef DEBUGGER_SUPPORT
   if(!myDataBusLocked)
@@ -243,7 +246,7 @@ void System::poke(uInt16 addr, uInt8 value)
   PageAccess& access = myPageAccessTable[page];
   
   // See if this page uses direct accessing or not 
-  if(access.directPokeBase != 0)
+  if(access.directPokeBase)
   {
     // Since we have direct access to this poke, we can dirty its page
     *(access.directPokeBase + (addr & myPageMask)) = value;
@@ -261,6 +264,34 @@ void System::poke(uInt16 addr, uInt8 value)
     myDataBusState = value;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 System::getAccessFlags(uInt16 addr)
+{
+#ifdef DEBUGGER_SUPPORT
+  PageAccess& access = myPageAccessTable[(addr & myAddressMask) >> myPageShift];
+
+  if(access.codeAccessBase)
+    return *(access.codeAccessBase + (addr & myPageMask));
+  else
+    return access.device->getAccessFlags(addr);
+#else
+  return 0;
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void System::setAccessFlags(uInt16 addr, uInt8 flags)
+{
+#ifdef DEBUGGER_SUPPORT
+  PageAccess& access = myPageAccessTable[(addr & myAddressMask) >> myPageShift];
+
+  if(access.codeAccessBase)
+    *(access.codeAccessBase + (addr & myPageMask)) |= flags;
+  else
+    access.device->setAccessFlags(addr, flags);
+#endif
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void System::lockDataBus()
 {
