@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ContextMenu.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: ContextMenu.cxx 2090 2010-08-09 17:02:36Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -24,6 +24,7 @@
 #include "FrameBuffer.hxx"
 #include "Dialog.hxx"
 #include "DialogContainer.hxx"
+#include "ScrollBarWidget.hxx"
 #include "ContextMenu.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -37,6 +38,9 @@ ContextMenu::ContextMenu(GuiObject* boss, const GUI::Font& font,
     _selectedOffset(0),
     _selectedItem(-1),
     _showScroll(false),
+    _isScrolling(false),
+    _scrollUpColor(kColor),
+    _scrollDnColor(kColor),
     _font(&font),
     _cmd(cmd),
     _xorig(0),
@@ -69,6 +73,10 @@ void ContextMenu::addItems(const StringMap& items)
   _x = _y = 0;
   _w = maxwidth + 10;
   _h = 1;  // recalculate this in ::recalc()
+
+  _scrollUpColor = _firstEntry > 0 ? kScrollColor : kColor;
+  _scrollDnColor = (_firstEntry + _numEntries < (int)_entries.size()) ?
+      kScrollColor : kColor;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -104,7 +112,7 @@ void ContextMenu::recalc(const GUI::Rect& image)
 {
   // Now is the time to adjust the height
   // If it's higher than the screen, we need to scroll through
-  int maxentries = (image.height() - 4) / _rowHeight;
+  int maxentries = BSPF_min(18, (image.height() - 4) / _rowHeight);
   if((int)_entries.size() > maxentries)
   {
     // We show two less than the max, so we have room for two scroll buttons
@@ -118,6 +126,7 @@ void ContextMenu::recalc(const GUI::Rect& image)
     _h = _entries.size() * _rowHeight + 4;
     _showScroll = false;
   }
+  _isScrolling = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -132,19 +141,22 @@ void ContextMenu::setSelected(int item)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::setSelected(const string& tag, const string& defaultTag)
 {
-  for(unsigned int item = 0; item < _entries.size(); ++item)
+  if(tag != "")  // indicates that the defaultTag should be used instead
   {
-    if(BSPF_strcasecmp(_entries[item].second.c_str(), tag.c_str()) == 0)
-    {
-      setSelected(item);
-      return;
+    for(unsigned int item = 0; item < _entries.size(); ++item)
+    { 
+      if(BSPF_equalsIgnoreCase(_entries[item].second, tag))
+      {
+        setSelected(item);
+        return;
+      }
     }
   }
 
   // If we get this far, the value wasn't found; use the default value
   for(unsigned int item = 0; item < _entries.size(); ++item)
   {
-    if(BSPF_strcasecmp(_entries[item].second.c_str(), defaultTag.c_str()) == 0)
+    if(BSPF_equalsIgnoreCase(_entries[item].second, defaultTag))
     {
       setSelected(item);
       return;
@@ -185,12 +197,17 @@ const string& ContextMenu::getSelectedTag() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::handleMouseDown(int x, int y, int button, int clickCount)
 {
+  // Compute over which item the mouse is...
+  int item = findItem(x, y);
+
   // Only do a selection when the left button is in the dialog
   if(button == 1)
   {
-    x += getAbsX(); y += getAbsY();
-    if(x >= _x && x <= _x+_w && y >= _y && y <= _y+_h)
+    if(item != -1)
+    {
+      _isScrolling = _showScroll && ((item == 0) || (item == _numEntries+1));
       sendSelection();
+    }
     else
       close();
   }
@@ -208,6 +225,26 @@ void ContextMenu::handleMouseMoved(int x, int y, int button)
   drawCurrentSelection(item);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool ContextMenu::handleMouseClicks(int x, int y, int button)
+{
+  // Let continuous mouse clicks come through, as the scroll buttons need them
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ContextMenu::handleMouseWheel(int x, int y, int direction)
+{
+  // Wheel events are only relevant in scroll mode
+  if(_showScroll)
+  {
+    if(direction < 0)
+      scrollUp(ScrollBarWidget::getWheelLines());
+    else if(direction > 0)
+      scrollDown(ScrollBarWidget::getWheelLines());
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::handleKeyDown(int ascii, int keycode, int modifiers)
 {
@@ -288,6 +325,8 @@ void ContextMenu::sendSelection()
       return scrollUp();
     else if(_selectedOffset == _numEntries+1) // scroll down
       return scrollDown();
+    else if(_isScrolling)
+      return;
     else
       item--;
   }
@@ -344,23 +383,30 @@ void ContextMenu::moveDown()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::scrollUp()
+void ContextMenu::scrollUp(int distance)
 {
-  if(_firstEntry > 0)
-  {
-    _firstEntry--;
-    setDirty();
-  }
+  if(_firstEntry == 0)
+    return;
+
+  _firstEntry = BSPF_max(_firstEntry - distance, 0);
+  _scrollUpColor = _firstEntry > 0 ? kScrollColor : kColor;
+  _scrollDnColor = kScrollColor;
+
+  setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::scrollDown()
+void ContextMenu::scrollDown(int distance)
 {
-  if(_firstEntry + _numEntries < (int)_entries.size())
-  {
-    _firstEntry++;
-    setDirty();
-  }
+  int max_offset = _entries.size() - _numEntries;
+  if(_firstEntry == max_offset)
+    return;
+
+  _firstEntry = BSPF_min(_firstEntry + distance, max_offset);
+  _scrollUpColor = kScrollColor;
+  _scrollDnColor = (_firstEntry < max_offset) ? kScrollColor : kColor;
+
+  setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -406,7 +452,7 @@ void ContextMenu::drawDialog()
     if(_showScroll)
     {
       s.hLine(x, y+_rowHeight-1, w+2, kShadowColor);
-      s.drawBitmap(up_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, kScrollColor, 8);
+      s.drawBitmap(up_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollUpColor, 8);
       y += _rowHeight;
       offset--;
     }
@@ -424,7 +470,7 @@ void ContextMenu::drawDialog()
     if(_showScroll)
     {
       s.hLine(x, y, w+2, kShadowColor);
-      s.drawBitmap(down_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, kScrollColor, 8);
+      s.drawBitmap(down_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollDnColor, 8);
     }
 
     s.addDirtyRect(_x, _y, _w, _h);
