@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Launcher.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: Launcher.cxx 2075 2010-07-22 15:41:46Z stephena $
 //============================================================================
 
 #include "LauncherDialog.hxx"
@@ -50,7 +50,7 @@ Launcher::~Launcher()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Launcher::initializeVideo()
+FBInitStatus Launcher::initializeVideo()
 {
   string title = string("Stella ") + STELLA_VERSION;
   return myOSystem->frameBuffer().initialize(title, myWidth, myHeight);
