@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: VideoDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: VideoDialog.cxx 2155 2010-10-18 18:39:57Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -31,6 +31,7 @@
 #include "EditTextWidget.hxx"
 #include "PopUpWidget.hxx"
 #include "Console.hxx"
+#include "TIA.hxx"
 #include "Settings.hxx"
 #include "StringList.hxx"
 #include "Widget.hxx"
@@ -58,7 +59,7 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
 
   // Set real dimensions
   _w = BSPF_min(52 * fontWidth + 10, max_w);
-  _h = BSPF_min(15 * (lineHeight + 4) + 10, max_h);
+  _h = BSPF_min(14 * (lineHeight + 4) + 10, max_h);
 
   // The tab widget
   xpos = ypos = 5;
@@ -74,8 +75,8 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
   // Video renderer
   new StaticTextWidget(myTab, font, xpos + (lwidth-fwidth), ypos, fwidth,
                        fontHeight, "Renderer:", kTextAlignLeft);
-  myRenderer = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                  pwidth, fontHeight, "");
+  myRenderer = new StaticTextWidget(myTab, font, xpos+lwidth, ypos,
+                                  fwidth, fontHeight, "", kTextAlignLeft);
   ypos += lineHeight + 4;
 
   items.clear();
@@ -163,20 +164,22 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
     new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                      "Framerate: ", lwidth, kFrameRateChanged);
   myFrameRateSlider->setMinValue(0); myFrameRateSlider->setMaxValue(300);
+  myFrameRateSlider->setStepValue(10);
   wid.push_back(myFrameRateSlider);
   myFrameRateLabel =
     new StaticTextWidget(myTab, font, xpos + myFrameRateSlider->getWidth() + 4,
-                         ypos + 1, fontWidth * 3, fontHeight, "", kTextAlignLeft);
+                         ypos + 1, fontWidth * 4, fontHeight, "", kTextAlignLeft);
   myFrameRateLabel->setFlags(WIDGET_CLEARBG);
 
   // Add message concerning usage
-  ypos = myTab->getHeight() - 5 - 2*fontHeight - 10;
-  new StaticTextWidget(myTab, font, 10, ypos,
+  const GUI::Font& infofont = instance().infoFont();
+  ypos = myTab->getHeight() - 5 - fontHeight - infofont.getFontHeight()- 10;
+  new StaticTextWidget(myTab, infofont, 10, ypos,
         font.getStringWidth("(*) Requires application restart"), fontHeight,
         "(*) Requires application restart", kTextAlignLeft);
 
   // Move over to the next column
-  xpos += myNAspectRatioSlider->getWidth() + myNAspectRatioLabel->getWidth() + 10;
+  xpos += myNAspectRatioSlider->getWidth() + myNAspectRatioLabel->getWidth() + 14;
   ypos = 10;
 
   // Fullscreen
@@ -224,7 +227,7 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
 
   // Center window (in windowed mode)
   myCenterCheckbox = new CheckboxWidget(myTab, font, xpos, ypos,
-                                        "Center window (*)");
+                                        "Center window");
   wid.push_back(myCenterCheckbox);
   ypos += lineHeight + 4;
 
@@ -344,6 +347,9 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
   myGrabmouseCheckbox->clearFlags(WIDGET_ENABLED);
   myCenterCheckbox->clearFlags(WIDGET_ENABLED);
 #endif
+#if !(defined(BSPF_WIN32) || (defined(BSPF_UNIX) && defined(HAVE_X11)))
+  myCenterCheckbox->clearFlags(WIDGET_ENABLED);
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -357,7 +363,7 @@ void VideoDialog::loadConfig()
   bool gl = (instance().frameBuffer().type() == kGLBuffer);
 
   // Renderer settings
-  myRenderer->setEditString(gl ? "OpenGL" : "Software");
+  myRenderer->setLabel(gl ? "OpenGL" : "Software");
   myRendererPopup->setSelected(
     instance().settings().getString("video"), "soft");
 
@@ -397,22 +403,23 @@ void VideoDialog::loadConfig()
   myPAspectRatioLabel->setLabel(instance().settings().getString("gl_aspectp"));
   myPAspectRatioLabel->setEnabled(gl);
 
-  // Framerate (0 or -1 means disabled)
+  // Framerate (0 or -1 means automatic framerate calculation)
   int rate = instance().settings().getInt("framerate");
   myFrameRateSlider->setValue(rate < 0 ? 0 : rate);
-  myFrameRateLabel->setLabel(rate < 0 ? "0" :
+  myFrameRateLabel->setLabel(rate <= 0 ? "Auto" :
     instance().settings().getString("framerate"));
 
   // Fullscreen
   const string& fullscreen = instance().settings().getString("fullscreen");
   myFullscreenPopup->setSelected(fullscreen, "0");
-  handleFullscreenChange(fullscreen == "1");
+  handleFullscreenChange(fullscreen == "0" || fullscreen == "1");
 
   // PAL color-loss effect
   myColorLossCheckbox->setState(instance().settings().getBool("colorloss"));
 
-  // GL stretch setting (item is enabled/disabled in ::handleFullscreenChange)
+  // GL stretch setting (GL mode only)
   myGLStretchCheckbox->setState(instance().settings().getBool("gl_fsmax"));
+  myGLStretchCheckbox->setEnabled(gl);
 
   // Use sync to vertical blank (GL mode only)
   myUseVSyncCheckbox->setState(instance().settings().getBool("gl_vsync"));
@@ -559,7 +566,7 @@ void VideoDialog::setDefaults()
   myPAspectRatioSlider->setValue(100);
   myPAspectRatioLabel->setLabel("100");
   myFrameRateSlider->setValue(0);
-  myFrameRateLabel->setLabel("0");
+  myFrameRateLabel->setLabel("Auto");
 
   myFullscreenPopup->setSelected("0", "");
   myColorLossCheckbox->setState(false);
@@ -567,7 +574,7 @@ void VideoDialog::setDefaults()
   myUseVSyncCheckbox->setState(true);
   myGrabmouseCheckbox->setState(false);
   myUIMessagesCheckbox->setState(true);
-  myCenterCheckbox->setState(true);
+  myCenterCheckbox->setState(false);
   myFastSCBiosCheckbox->setState(false);
 
   myTexturePopup->setSelected("off", "");
@@ -576,7 +583,7 @@ void VideoDialog::setDefaults()
   myPhosphorCheckbox->setState(false);
 
   // Make sure that mutually-exclusive items are not enabled at the same time
-  handleFullscreenChange(false);
+  handleFullscreenChange(true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -584,11 +591,6 @@ void VideoDialog::handleFullscreenChange(bool enable)
 {
 #ifdef WINDOWED_SUPPORT
   myFSResPopup->setEnabled(enable);
-
-  // GL stretch is only enabled in OpenGL mode
-  myGLStretchCheckbox->setEnabled(
-    enable && instance().frameBuffer().type() == kGLBuffer);
-
   _dirty = true;
 #endif
 }
@@ -617,11 +619,14 @@ void VideoDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kFrameRateChanged:
-      myFrameRateLabel->setValue(myFrameRateSlider->getValue());
+      if(myFrameRateSlider->getValue() == 0)
+        myFrameRateLabel->setLabel("Auto");
+      else
+        myFrameRateLabel->setValue(myFrameRateSlider->getValue());
       break;
 
     case kFullScrChanged:
-      handleFullscreenChange(myFullscreenPopup->getSelectedTag() == "1");
+      handleFullscreenChange(myFullscreenPopup->getSelectedTag() != "-1");
       break;
 
     default:
