@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Widget.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: Widget.cxx 2097 2010-08-14 01:01:27Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -610,6 +610,18 @@ void SliderWidget::handleMouseUp(int x, int y, int button, int clickCount)
   _isDragging = false;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SliderWidget::handleMouseWheel(int x, int y, int direction)
+{
+  if(isEnabled())
+  {
+    if(direction < 0)
+      handleEvent(Event::UIUp);
+    else if(direction > 0)
+      handleEvent(Event::UIDown);
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool SliderWidget::handleEvent(Event::Type e)
 {
@@ -618,13 +630,11 @@ bool SliderWidget::handleEvent(Event::Type e)
 
   switch(e)
   {
-    case Event::UILeft:
     case Event::UIDown:
     case Event::UIPgDown:
       setValue(_value - _stepValue);
       break;
   
-    case Event::UIRight:
     case Event::UIUp:
     case Event::UIPgUp:
       setValue(_value + _stepValue);
@@ -672,8 +682,9 @@ int SliderWidget::valueToPos(int value)
 {
   if(value < _valueMin)      value = _valueMin;
   else if(value > _valueMax) value = _valueMax;
-
-  return ((_w - _labelWidth - 4) * (value - _valueMin) / (_valueMax - _valueMin));
+  int range = BSPF_max(_valueMax - _valueMin, 1);  // don't divide by zero
+  
+  return ((_w - _labelWidth - 4) * (value - _valueMin) / range);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
